#include <LiquidCrystal.h>
#include <LCDKeypad.h>
#include <OneWire.h>
#include <DallasTemperature.h>
//pin definitions
#define T1 A1 //DS18B20 sensor
#define Trigger 2
#define Echo 3
//peripherals
OneWire ds(T1); //attach sensor T1 
DallasTemperature sensors(&ds);
LCDKeypad lcd; //attach LCD keypad shield
//LCD button
int btn;
//cm -in
bool cm = true;
//second display
bool disp2 = false;
bool rel = false;
//initial backlight
unsigned char bckl=150;
//T1 temperature variable
float temp1;
float sound_speed;
unsigned long pulselength;
float distancecm, distancein, relcm, relin;
void setup()
{
  pinMode(Trigger, OUTPUT);
  pinMode(Echo, INPUT);
  pinMode(10, OUTPUT);
  analogWrite(10, bckl);
  lcd.begin(16,2);
  lcd.clear();   
  lcd.print("Echolocator");  
  lcd.setCursor(0,1);
  lcd.print("      v1.0    ");
  sensors.begin();//IC default 9 bit
  delay(500);

}
void readTemperatures()
{
 sensors.requestTemperatures(); // Send the command to get temperatures
 delay(500);
 temp1 = sensors.getTempCByIndex(0);

} 
void calculateSoundSpeed()
{
  //CAIR = 20.0457sqrt(273.15+T) m/s
  sound_speed = 20.0457*sqrt(273.15+temp1)/10000.0; //cm/us
  
}
void getDistance()
{
  digitalWrite(Trigger, LOW);
  delayMicroseconds(5);
  digitalWrite(Trigger, HIGH);
  delayMicroseconds(10);
  digitalWrite(Trigger, LOW);
  //read signal from Echo pin
  pulselength =  pulseIn(Echo, HIGH); //microseconds
  //calculate distance
  distancecm = sound_speed *  (pulselength/2);
  distancein = distancecm * 0.39370; 
}

void displayDistance()
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("T:");
  lcd.print(temp1,1);
  lcd.print("C; ");

  lcd.setCursor(0,1); 
  lcd.print("D:");
  if (cm)
  {
  lcd.print(distancecm,1);
  lcd.print("cm; ");
  }
  else{
  lcd.print(distancein,1);
  lcd.print("in; ");
  }
}

void display2()
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("p:");
  lcd.print(pulselength);
  lcd.print("us; ");

  lcd.setCursor(0,1); 
  lcd.print("V:");
  if (cm)
  {
  lcd.print(sound_speed*10000,1);
  lcd.print("m/s; ");
  }
  else{
  lcd.print(sound_speed*10000*39.370,1);
  lcd.print("in/s; ");
  }
}

void displayRel()
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Relative");

  lcd.setCursor(0,1); 
  if (cm)
  {
  lcd.print(distancecm-relcm,1);
  lcd.print("cm; ");
  }
  else{
  lcd.print(distancein-relin,1);
  lcd.print("in; ");
  }
}

void readButtons()
{
  btn=lcd.button();
if (btn != KEYPAD_NONE)
{
   //wait for button release
 while(lcd.button()!=KEYPAD_NONE)
 {
 }
}
if (btn == KEYPAD_UP)
{
  cm = true;
}else if (btn==KEYPAD_DOWN)
{
  cm = false;
}else if (btn==KEYPAD_LEFT)
{
  disp2 = false;
}else if (btn==KEYPAD_RIGHT)
{
  disp2 = true;
}else if (btn==KEYPAD_SELECT)
{
  rel = !rel;
  disp2 = false;
  relcm = distancecm;
  relin = distancein;
}  
}
void loop(){
  readTemperatures();
  calculateSoundSpeed();
  getDistance();
  readButtons();
  if (disp2)
  {
      display2();
  }else if (rel){
   displayRel();
  }else{
    displayDistance(); 
  }

  delay(100);
}

